"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FabricApiClient = void 0;
const common_1 = require("@ts-core/common");
const fabric_network_1 = require("fabric-network");
const _ = require("lodash");
const FabricUtil_1 = require("./FabricUtil");
const FabricContractQSCC_1 = require("./contract/FabricContractQSCC");
const FabricConnectionFileParser_1 = require("./parser/FabricConnectionFileParser");
class FabricApiClient extends common_1.LoggerWrapper {
    static parseBlock(block) {
        let item = block;
        item.hash = FabricUtil_1.FabricUtil.fromUintArray(block.header.data_hash);
        item.number = Number(block.header.number);
        item.date = FabricApiClient.getBlockDate(block);
        return item;
    }
    static getBlockDate(block) {
        if (_.isNil(block.data) || _.isEmpty(block.data.data)) {
            return null;
        }
        for (let data of block.data.data) {
            if (_.isNil(data) || _.isNil(data.payload) || _.isNil(data.payload.header) || _.isNil(data.payload.header.channel_header)) {
                continue;
            }
            return new Date(data.payload.header.channel_header.timestamp);
        }
        return null;
    }
    static createConnection(settings, wallet) {
        return __awaiter(this, void 0, void 0, function* () {
            let gatewayConfig = null;
            if (_.isString(settings.fabricConnectionSettings)) {
                gatewayConfig = JSON.parse(FabricConnectionFileParser_1.FabricConnectionFileParser.load(settings.fabricConnectionSettings));
            }
            else {
                gatewayConfig = settings.fabricConnectionSettings;
            }
            if (_.isNil(wallet)) {
                wallet = yield FabricApiClient.createWallet(settings);
            }
            let gatewayOptions = {
                wallet,
                identity: settings.fabricIdentity,
                clientTlsIdentity: settings.fabricTlsIdentity,
                discovery: { enabled: settings.fabricIsDiscoveryEnabled, asLocalhost: settings.fabricIsDiscoveryAsLocalhost }
            };
            let gateway = new fabric_network_1.Gateway();
            yield gateway.connect(gatewayConfig, gatewayOptions);
            let network = yield gateway.getNetwork(settings.fabricNetworkName);
            let channel = network.getChannel();
            let contract = network.getContract(settings.fabricChaincodeName);
            return { gateway, wallet, network, channel, contract };
        });
    }
    static createWallet(settings) {
        return __awaiter(this, void 0, void 0, function* () {
            let item = yield fabric_network_1.Wallets.newInMemoryWallet();
            let identity = {
                type: 'X.509',
                mspId: settings.fabricIdentityMspId,
                credentials: {
                    privateKey: settings.fabricIdentityPrivateKey,
                    certificate: settings.fabricIdentityCertificate
                }
            };
            item.put(settings.fabricIdentity, identity);
            if (!_.isNil(settings.fabricTlsIdentity)) {
                let tlsIdentity = {
                    type: 'X.509',
                    mspId: settings.fabricTlsIdentityMspId,
                    credentials: {
                        privateKey: settings.fabricTlsIdentityPrivateKey,
                        certificate: settings.fabricTlsIdentityCertificate
                    }
                };
                item.put(settings.fabricTlsIdentity, tlsIdentity);
            }
            return item;
        });
    }
    constructor(logger, settings) {
        super(logger);
        this.settings = settings;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (_.isNil(this.settings)) {
                throw new common_1.ExtendedError(`Unable to connect: settings is nil`);
            }
            if (!_.isNil(this.connectionPromise)) {
                return this.connectionPromise.promise;
            }
            this.connectionPromise = common_1.PromiseHandler.create();
            this.reconnect();
            return this.connectionPromise.promise;
        });
    }
    disconnect(error) {
        if (!_.isNil(this.connectionPromise)) {
            this.connectionPromise.reject(error);
            this.connectionPromise = null;
        }
        this.connection = null;
    }
    destroy() {
        if (this.isDestroyed) {
            return;
        }
        super.destroy();
        this.disconnect();
        this.observer.complete();
        this.observer = null;
    }
    reconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.debug(`Connecting to Fabric "${this.settings.fabricIdentity}:${this.settings.fabricNetworkName}:${this.settings.fabricChaincodeName}"`);
            try {
                this.connection = yield FabricApiClient.createConnection(this.settings);
            }
            catch (error) {
                this.connectErrorHandler(common_1.ExtendedError.create(error, common_1.ExtendedError.DEFAULT_ERROR_CODE));
            }
        });
    }
    connectCompleteHandler() {
        if (!_.isNil(this.connectionPromise)) {
            this.connectionPromise.resolve();
        }
    }
    connectErrorHandler(error) {
        this.disconnect(error);
    }
    get connection() {
        return this._connection;
    }
    set connection(value) {
        if (value === this._connection) {
            return;
        }
        if (!_.isNil(this._connection)) {
            this._connection.gateway.disconnect();
        }
        this._connection = value;
        this._isConnected = !_.isNil(this._connection);
        this._qsccContract = !_.isNil(this._connection) ? new FabricContractQSCC_1.FabricContractQSCC(this) : null;
        if (this._isConnected) {
            this.connectCompleteHandler();
        }
        else {
            this.connectErrorHandler();
        }
    }
    get isConnected() {
        return this._isConnected;
    }
    get network() {
        return !_.isNil(this.connection) ? this.connection.network : null;
    }
    get channel() {
        return !_.isNil(this.connection) ? this.connection.channel : null;
    }
    get contract() {
        return !_.isNil(this.connection) ? this.connection.contract : null;
    }
    get gateway() {
        return !_.isNil(this.connection) ? this.connection.gateway : null;
    }
    get qsccContract() {
        return this._qsccContract;
    }
}
exports.FabricApiClient = FabricApiClient;
